
package
{
	import flash.display.*;
	import flash.events.*;
	import flash.utils.getTimer;
	import gs.*;

	public class Game extends Sprite
	{
		public var player: Player;
		
		public var invader: Invader;
		
		public var shields: Array;
		
		public var bullets: Array;
		public var debris: Array;
		
		private var time: int;
		
		public function Game ()
		{
			shields = new Array();
			
			for (var i: int = 0; i < 4; i++) {
				var shield: Shield = new Shield(this, i);
				shields.push(shield);
				
				addChild(shield);
			}
			
			player = new Player(this);
			invader = new Invader(this);
			
			addChild(invader);
			addChild(player);
			
			bullets = new Array();
			debris = new Array();
			
			addEventListener(Event.ADDED_TO_STAGE, init);
			addEventListener(Event.REMOVED_FROM_STAGE, uninit);
		}
		
		public function init (param : * = 0) : void
		{
			time = getTimer();
			
			stage.addEventListener(Event.ENTER_FRAME, onEnterFrame);
		}
		
		public function uninit (param : * = 0) : void
		{
			stage.removeEventListener(Event.ENTER_FRAME, onEnterFrame);
		}
		
		private function onEnterFrame(e:Event):void
		{
			var dt:int = getTimer() - time;
			
			while (dt >= 20) {
				time += 20;
				dt -= 20;
				
				update();
			}
		}
		
		public function update (): void
		{
			player.update();
			
			invader.update();
			
			var shield: Shield;
			
			for each (shield in shields) {
				shield.update();
			}
			
			for each (var b: Bullet in bullets)
			{
				b.update();
				invader.check(b);
				
				for each (shield in shields) {
					shield.check(b);
				}
			}
			
			for each (var p: DebrisParticle in debris)
			{
				for (var i: int = 0; i < 2; i++) {
					p.update();
					
					for each (shield in shields) {
						shield.check(p);
					}
					
					if (player.hitTestPoint(p.x, p.y, true)) {//(PixelPerfectCollisionDetection.isColliding(player, p, this, true))
						player.explode();
						
						TweenLite.delayedCall(2.0, function (): void { Main.screen = new MainMenu(); });
						
						return;
					}
				}
			}
			
			bullets = bullets.filter(removeBulletsFilter);
			debris = debris.filter(removeDebrisFilter);
		}
		
		public function addBullet (x: Number, y:Number): void
		{
			var bullet: Bullet = new Bullet(x, y);
			
			bullets.push(bullet);
			
			addChild(bullet);
		}
		
		public function addDebris (particle: DebrisParticle): void
		{
			debris.push(particle);
			
			addChild(particle);
		}
		
		private function removeBulletsFilter (p: Bullet, index: int, arr: Array): Boolean
		{
			if (p.y < -50) {
				removeChild(p);
				return false;
			} else {
				return true;
			}
		}
		
		private function removeDebrisFilter (p: DebrisParticle, index: int, arr: Array): Boolean
		{
			if (p.y > 500) {
				removeChild(p);
				return false;
			} else {
				return true;
			}
		}
	}

}


