
package
{
	import flash.display.*;
	import flash.events.*;
	import flash.ui.*;
	import flash.utils.getTimer;
	import flash.media.*;
	import gs.*;

	public class Player extends Sprite
	{
		public var game: Game;
		
		[Embed(source="images/player.png")]
		public static var playerGfx: Class;
		
		[Embed(source="audio/fire.mp3")]
		public static var fireSfx: Class;
		
		[Embed(source="audio/die.mp3")]
		public static var dieSfx: Class;
		
		public var image: Bitmap;
		
		private var fireSound: Sound;
		private var dieSound: Sound;
		
		private var left: Boolean;
		private var right: Boolean;
		private var fire: Boolean;
		
		private var fireTimer: int = 0;
		
		public function Player (_game: Game)
		{
			game = _game;
			
			x = 300;
			y = 430;
			
			image = new playerGfx();
			addChild(image);
			
			fireSound = new fireSfx();
			dieSound = new dieSfx();
			
			addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		public function update (): void
		{
			if (left)
			{
				x -= 4;
				
				if (x < 0)
				{
					x = 0;
				}
			}
			
			if (right)
			{
				x += 4;
				
				if (x + width > 640)
				{
					x = 640 - width;
				}
			}
			
			if (fireTimer == 0)
			{
				if (fire)
				{
					game.addBullet(x + width / 2, y);
					fireTimer = 25;
					
					//fireSound.play();
				}
			} else {
				fireTimer -= 1;
			}
		}
		
		public function explode (): void {
			dieSound.play();
			for (var i: int = 0; i < 13; i++) {
				for (var j: int = 0; j < 8; j++) {
					var colour: uint = image.bitmapData.getPixel32(i*4 + 2, j*4 + 2);
					
					var g: uint = (colour >> 8) & 0xFF;
					
					if (g > 0) {
						var particle: DebrisParticle = new DebrisParticle(x + i*4 + 2, y + j*4 + 2, (i - 7) / 4, -2);
						
						particle.graphics.beginFill(colour);
						particle.graphics.drawRect(-2, -2, 4, 4);
						particle.graphics.endFill();
						
						game.addDebris(particle);
					}
				}
			}
			
			y = -500;
		}
		
		private function init (param : * = null): void
		{
			stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			stage.addEventListener(KeyboardEvent.KEY_UP, onKeyUp);
		}
		
		private function onKeyDown (e: KeyboardEvent): void
		{
			if (e.keyCode == Keyboard.LEFT)
			{
				left = true;
			}
			else if (e.keyCode == Keyboard.RIGHT)
			{
				right = true;
			}
			else
			{
				fire = true;
			}
		}
		
		private function onKeyUp (e: KeyboardEvent): void
		{
			if (e.keyCode == Keyboard.LEFT)
			{
				left = false;
			}
			else if (e.keyCode == Keyboard.RIGHT)
			{
				right = false;
			}
			else
			{
				fire = false;
			}
		}
		
	}

}


